{
Client Access Objects 1.5

caoDataQueue component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

An AS/400 data queue is an object used to communicate and store data.  It can
be used by several programs in a job or between jobs.  The cwbDataQueue
automation object can be used to read and write data between a personal
computer and a data queue on an AS/400 system.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoDataQueue;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  caoErrorMessage, caoDataQueueAttr, caoDataQueueDataAttr,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  // caoDataQueue exception object

  EcaoDataQueue = class(Exception);

  TcaoDataQueue = class(TComponent)
  private
    { Private declarations }
    FReturnCode: integer;
    FcaoErrorMessage: TcaoErrorMessage;
    FcaoDataQueueAttr: TcaoDataQueueAttr;
    FcaoDataQueueDataAttr: TcaoDataQueueDataAttr;
    FcwbDataQueue: cwbDataQueue;
    FOnOpen: TNotifyEvent;
    FOnClose: TNotifyEvent;
    function GetLibraryName: WideString;
    function GetSystemName: WideString;
    function GetQueueName: WideString;
  protected
    { Protected declarations }
    function ResultCodeText: string; virtual;
    function GetErrMsg: string; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure AsyncRead(waitTime: Integer; var readHandle: Integer); virtual;
    procedure Cancel(readHandle: Integer); virtual;
    procedure CheckData(readHandle: Integer; var dataBuffer: OleVariant; var dataLength: Integer); virtual;
    procedure Clear(key: OleVariant; keyLength: Smallint); virtual;
    procedure Close; virtual;
    procedure CreateQ(const queue, libraryName, systemName: WideString); virtual;
    procedure Delete(const queue, libraryName, systemName: WideString); virtual;
    procedure GetQueueAttr; virtual;
    procedure Open(const queue, libraryName, systemName: WideString); virtual;
    procedure Peek(var dataBuffer: OleVariant; var dataLength: Integer; waitTime: Integer); virtual;
    procedure Read(var dataBuffer: OleVariant; var dataLength: Integer; waitTime: Integer); virtual;
    procedure Write(var dataBuffer: OleVariant; dataLength: Integer; commit: WordBool); virtual;
  published
    { Published declarations }
    property ReturnCode: integer read FReturnCode default cwbOK;
    property systemName: WideString read GetSystemName;
    property libraryName: WideString read GetLibraryName;
    property queueName: WideString read GetQueueName;
    property errorMessage: TcaoErrorMessage read FcaoErrorMessage write FcaoErrorMessage;
    property dataQueueAttr: TcaoDataQueueAttr read FcaoDataQueueAttr write FcaoDataQueueAttr;
    property dataQueueDataAttr: TcaoDataQueueDataAttr read FcaoDataQueueDataAttr write FcaoDataQueueDataAttr;
    property OleObject: cwbDataQueue read FcwbDataQueue;
    property OnOpen: TNotifyEvent read FOnOpen write FOnOpen;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
  end;

implementation

{
Create OLE object and initilize properties.
}

constructor TcaoDataQueue.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FcwbDataQueue := CocwbDataQueue.Create;
end;

{
Return formatted result code text.
}

function TcaoDataQueue.ResultCodeText: string;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
Return error mesage or formatted result code text.
}

function TcaoDataQueue.GetErrMsg: string;
var
  S: string;
begin
  S := FcaoErrorMessage.GetErrMsg;
  if S = '' then
    S := ResultCodeText;
  Result := S
end;

{
This property contains the name of the AS/400 library which contains the data
queue to be accessed.  This is a read-only property.
}

function TcaoDataQueue.GetLibraryName: WideString;
begin
  Result := FcwbDataQueue.libraryName
end;

{
This property contains the AS/400 system name on which the data queue to be
accessed resides.  This is a read-only property.

The system name may be in one of the following formats:

	Fully qualified SNA network name (APPN.SYSTEM)
	TCP/IP network name (SYSTEM.NETWORK.COM)
	System name without a network identifier (SYSTEM)
	System alias name (SYSALIAS)

The format must match the format used to configure the connection.
}

function TcaoDataQueue.GetSystemName: WideString;
begin
  Result := FcwbDataQueue.systemName
end;

{
This property contains the name of the data queue to be accessed.  This is a
read-only property.
}

function TcaoDataQueue.GetQueueName: WideString;
begin
  Result := FcwbDataQueue.queueName
end;

{
Handle caoErrorMessage, caoDataQueueAttr or caoDataQueueDataAttr component
being removed.
}

procedure TcaoDataQueue.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FcaoErrorMessage then
      FcaoErrorMessage := nil
    else
      if AComponent = FcaoDataQueueAttr then
        FcaoDataQueueAttr := nil
      else
        if AComponent = FcaoDataQueueDataAttr then
          FcaoDataQueueDataAttr := nil
  end
end;

{
This method attempts to read a record from the AS/400 data queue.  If the queue
is empty, it waits for data to be put on the queue, provided a non-zero
waitTime is specified.  To actually retrieve the data, a call to the
cwbdqCheckData method must be made, using the readHandle returned from this
call.  When the data is read, it is removed from the queue.

Parameters:

waitTime - Long

Length of time, in seconds, to wait for data if the data queue is empty.  Valid
values are 0 to 99,000 and -1.  Zero means return immediately.  -1 means wait
forever.

readHandle - Pointer to Long

A handle to be used on subsequent cwbdqCheckData calls.  It is used to identify
the cwbdqAsyncRead request for which data is to be retrieved.
}

procedure TcaoDataQueue.AsyncRead(waitTime: Integer; var readHandle: Integer);
begin
  FReturnCode := FcwbDataQueue.cwbdqAsyncRead(
  FcaoDataQueueDataAttr.OleObject,waitTime,readHandle,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

{
This method cancels a previously issued cwbdqAsyncRead request.  Even if the
wait time has not expired, the asynchronous read request no longer waits for
data to be put on the queue.

Parameters:

readHandle - Long

The handle returned by the cwbdqAsyncRead method call.
}

procedure TcaoDataQueue.Cancel(readHandle: Integer);
begin
  FReturnCode := FcwbDataQueue.cwbdqCancel(
  readHandle,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

{
This method  checks to see if data was returned from a previously issued
cwbdqAsyncRead method call.  If the return value is cwbdqNoData this request
can be made multiple times, with the same readHandle value, to continually
check for data.  If the return value is anything other than cwbdqNoData
(including cwbOK), the readHandle will be invalidated.

Parameters:

readHandle - Long

The handle returned by the cwbdqAsyncRead method call.

dataBuffer - Byte Array Variant

The buffer in which the data should be stored.  The maxRecordLength property of
the cwbDataQueueAttr object can be retrieved to help determine the size of the
buffer needed.  (This parameter should be passed as a SafeArray of unsigned
characters.)

dataLength - Pointer to Long

On output, it is the amount of data stored in dataBuffer.
}

procedure TcaoDataQueue.CheckData(readHandle: Integer; var dataBuffer: OleVariant; var dataLength: Integer);
begin
  FReturnCode := FcwbDataQueue.cwbdqCheckData(
  readHandle,dataBuffer,dataLength,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

{
This method removes all records from the AS/400 data queue.  If the queue is
keyed, only records for a particular key may be removed by specifying the key
and key length.  These values should be set to NULL and zero, respectively, if
you want to clear all messages from the queue.

Parameters:

key - Byte Array Variant

The key for which records are to be removed.  The key may contain embedded null
characters.  (This parameter should be passed as a SafeArray of unsigned
characters.)

keyLength - Short Integer

Length of key , in bytes.
}

procedure TcaoDataQueue.Clear(key: OleVariant; keyLength: Smallint);
begin
  FReturnCode := FcwbDataQueue.cwbdqClear(
  key,keyLength,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

{
This method ends the conversation with the AS/400 system.
}

procedure TcaoDataQueue.Close;
begin
  FReturnCode := FcwbDataQueue.cwbdqClose;
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg);
  if Assigned(OnClose) then
    OnClose(Self)
end;

{
This method  creates an AS/400 data queue object.  After the object is created,
it can be opened by using the cwbdqOpen method call.

Parameters:

queue - String

The name of the AS/400 data queue to be created.

libraryName - String

The name of the library in which the data queue should be created.

systemName - String

The name of the AS/400 system on which the library exists.
}

procedure TcaoDataQueue.CreateQ(const queue, libraryName, systemName: WideString);
begin
  FReturnCode := FcwbDataQueue.cwbdqCreate(
  queue,libraryName,systemName,FcaoDataQueueAttr.OleObject,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

{
This method removes all data from an AS/400 data queue and deletes the AS/400
data queue object.

Parameters:

queue - String

The name of the data queue to be deleted.

libraryName - String

The name of the library in which the data queue resides.

systemName - String

The name of the AS/400 system on which the library and data queue reside.
}

procedure TcaoDataQueue.Delete(const queue, libraryName, systemName: WideString);
begin
  FReturnCode := FcwbDataQueue.cwbdqDelete(
  queue,libraryName,systemName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

{
This method retrieves the attributes of a specified AS/400 data queue.
}

procedure TcaoDataQueue.GetQueueAttr;
begin
  FReturnCode := FcwbDataQueue.cwbdqGetQueueAttr(
  FcaoDataQueueAttr.OleObject,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

{
This method starts a conversation with the specified AS/400 system and
establishes a connection to the specified data queue.

Parameters:

queue - String

The name of the data queue object to be opened.

libraryName - String

The name of the library in which the data queue object resides.

systemName - String

The name of the AS/400 system on which the library and data queue reside.
}

procedure TcaoDataQueue.Open(const queue, libraryName, systemName: WideString);
begin
  FReturnCode := FcwbDataQueue.cwbdqOpen(
  queue,libraryName,systemName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg);
  if Assigned(OnOpen) then
    OnOpen(Self)
end;

{
This method reads a record from an AS/400 data queue.  When a record is peeked
from a data queue it remains on the data queue.  If the queue is empty, it will
wait for data to be put on the queue, provided a non-zero waitTime is
specified.

Parameters:

dataBuffer - Byte Array Variant

The buffer in which the data should be stored.  The maxRecordLength property of
the cwbDataQueueAttr object can be retrieved to help determine the size of the
buffer needed.  (This parameter should be passed as a SafeArray of unsigned
characters.)

dataLength - Pointer to Long

On output, it is the amount of data stored in dataBuffer.

waitTime - Long

Length of time, in seconds, to wait for data if the data queue is empty.  Valid
values are 0 to 99,000 and -1.  Zero means return immediately.  -1 means wait
forever.
}

procedure TcaoDataQueue.Peek(var dataBuffer: OleVariant; var dataLength: Integer; waitTime: Integer);
begin
  FReturnCode := FcwbDataQueue.cwbdqPeek(
  FcaoDataQueueDataAttr.OleObject,dataBuffer,dataLength,waitTime,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

{
This method reads a record from an AS/400 data queue.  When a record is read
from a data queue, it is removed from the data queue.  If the queue is empty,
it will wait for data to be put on the queue, provided a non-zero waitTime is
specified.

Parameters:

dataBuffer - Byte Array Variant

The buffer in which the data should be stored.  The maxRecordLength property of
the cwbDataQueueAttr object can be retrieved to help determine the size of the
buffer needed.  (This parameter should be passed as a SafeArray of unsigned
characters.)

dataLength - Pointer to Long

On output, it is the amount of data stored in dataBuffer.

waitTime - Long

Length of time, in seconds, to wait for data if the data queue is empty.  Valid
values are 0 to 99,000 and -1.  Zero means return immediately.  -1 means wait
forever.
}

procedure TcaoDataQueue.Read(var dataBuffer: OleVariant; var dataLength: Integer; waitTime: Integer);
begin
  FReturnCode := FcwbDataQueue.cwbdqRead(
  FcaoDataQueueDataAttr.OleObject,dataBuffer,dataLength,waitTime,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

{
This method  writes a record to an AS/400 data queue.

Parameters:

dataBuffer - Byte Array Variant

A buffer containing the data to write.  (This parameter should be passed as
SafeArray of unsigned characters.)

dataLength - Long

The length of the data contained in the dataBuffer.

commit - Boolean

Flag indicating whether or not the data must be enqueued before returning to
the caller.  If the value is true, the data is enqueued before returning.
}

procedure TcaoDataQueue.Write(var dataBuffer: OleVariant; dataLength: Integer; commit: WordBool);
begin
  FReturnCode := FcwbDataQueue.cwbdqWrite(
  FcaoDataQueueDataAttr.OleObject,dataBuffer,dataLength,commit,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueue.Create(GetErrMsg)
end;

end.
